--------------------------------------------------------
--  Paolo Santi - 2007                                --
--  Railway (il ritorno) 1.1                          --
--  Universita' di Padova                             --
--  Anno accademico 2006/2007                         --
--  Laurea Magistrale in Informatica                  --
--  Corso di Sistemi Concorrenti e Distribuiti        --
--                                                    --
--  Rielaborazione di un progetto di Alessandro Temil --
--                                                    --
--------------------------------------------------------
--  Copyright (C) 2006                                --
--  Alessandro Temil                                  --
--  Railway 1.0                                       --
--  Concurrent and Distributed Systems class project  --
--  Master Degree in Computer Science                 --
--  Academic year 2005/6                              --
--  Dept. of Pure and Applied Mathematics             --
--  University of Padua, Italy                        --
--                                                    --
--  This program is free software; you can            --
--  redistribute it and/or modify it under the terms  --
--  of the GNU General Public License as published    --
--  by the Free Software Foundation; either           --
--  version 2 of the License, or (at your option)     --
--  any later version.                                --
--------------------------------------------------------

with Railway;                     use Railway;
with Ada.Real_Time;               use Ada.Real_Time;
with Railway.Types;               use Railway.Types;

package Railway.Platform is

   type Platform_T;
   type Platform_Ref_T is access all Platform_T;

   type AllPlatforms_T is array(Platform_ID_T) of Platform_Ref_T;
   AllPlatformsReference  : access AllPlatforms_T := null;

   type Ticket_DB_T;
   type Ticket_DB_Ref_T is access all Ticket_DB_T;

   All_The_Ticket_DB_Refs : array(Range_Trains) of Ticket_DB_Ref_T :=
     (others => null);

   --  passenger travel definition: extended leg
   --
   --  an extended traveler's leg is derived from Travel_Leg_T
   --  adding starting and ending stages, starting and ending
   --  platforms
   type Travel_Ext_Leg_T is record
      --  train identifier (given)
      Train : Range_Trains;
      --  starting station of leg (given)
      Station_From  : Station_T;
      --  ending station of leg (given)
      Station_To    : Station_T;

      --  type of train IC/IR
      Train_Type : Train_Type_T;
      --  starting stage of leg (filled by Traveler's task)
      Stg_From      : Integer := 0;
      --  ending stage of leg (filled by Traveler's task)
      Stg_To        : Integer := 0;
      --  total number of train's stages (filled by Traveler's task)
      Stg_Tot       : Integer := 0;
      --  starting platform of leg (filled by Traveler's task)
      Plat_From     : Platform_ID_T := Null_Platform_ID;
      --  ending platform of leg (filled by Traveler's task)
      Plat_To       : Platform_ID_T := Null_Platform_ID;
   end record;

   --  a route is an array of legs
   type Travel_Ext_Route_T is array (Integer range <>) of Travel_Ext_Leg_T;
   type Travel_Ext_Route_Ref_T is access all Travel_Ext_Route_T;

   --  controls advancement in simulation Time
   Passed : Time_Span;

   protected type Platform_T
     (Name    : String_Ref_T;
      Station : String_Ref_T) is

      --  called by Trains
      --+ when they Arrive at a Station
      entry Train_Arrived
        (Train_Name : String;
         Train_Number : Range_Trains;
         Free_Seats : Integer;
         Seats : Integer;
         Destination : String);

      --  enqueues at the Platform
      entry Stopping
        (Train_Name : String;
         Train_Number : Range_Trains;
         Free_Seats : Integer;
         Seats : Integer;
         Destination : String);

      --  boards Passengers
      procedure Boarding
        (Train_Name : String;
         Next_Stage : Integer;
         Destination : String);

      --  Departs
      procedure Close_Doors
        (Train_Name : String;
         Free_Seats : out Integer;
         Destination : String);


      --  Passenger side

      --  the Passenger waits for the Train at the Platform
      entry Depart (Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

      --  the Passenger gets on the Train at the Platform
      entry GetOn (Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

      --  once on board, it's enqueued at the arriving Station to exit
      entry Arrive (Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

      --  or to change Train..
      entry Change_Train (Range_Trains)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

   private

      Stopped_Train : String_Ref_T := null;
      Stopped_Train_Number : Range_Trains;
      TrainBoarding : String_Ref_T := null;
      Train_Next_Stage : Integer;
      tFree_Seats : Integer;
      tSeats : Integer;

   end Platform_T;

   type blg_T is array (Integer range <>) of Integer;

   --  a ticket daTabase for one Train
   protected type Ticket_DB_T
     (Train_Name : String_Ref_T;
      Train_Number : Range_Trains;
      Route : Path_Ref_T;
      Routefirst : Integer;
      Routelast : Integer;
      NumberofSeats : Integer) is

        --  books one leg on one Train
      entry Book_Leg
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

        --  books one segment of the leg
      entry Book_Ticket (Routefirst..Routelast)
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

      --  releases a ticket once is used
      procedure Release_Ticket
        (Passenger_Route : Travel_Ext_Route_T;
         Travel_Index : in out Integer;
         Passenger_Name : String);

   private

      Available_Tickets: blg_T (Routefirst..Routelast) :=
        (others => NumberofSeats);

   end Ticket_DB_T;

end Railway.Platform;
